#include <stdio.h>
#include <stdlib.h>
#include "abb.h"
#include "fila.h"

void inicializar(abb **raiz)
{
  *raiz = NULL;
}

abb *pesquisar(abb *raiz, Tipo elem)
{
  if (raiz == NULL || elem == raiz->info)
    return raiz;
  else if (elem < raiz->info)
    return pesquisar(raiz->esq, elem);
  else
    return pesquisar(raiz->dir, elem);
}

abb *pesquisarIterativo(abb *raiz, Tipo elem)
{
  while (raiz != NULL)
  {
    if (elem == raiz->info)
      break;
    else if (elem < raiz->info)
      raiz = raiz->esq;
    else
      raiz = raiz->dir;
  }
  return raiz;
}

abb *pesquisarMinimo(abb *raiz)
{
  if (raiz == NULL || raiz->esq == NULL)
    return raiz;
  else
    return pesquisarMinimo(raiz->esq);
}

abb *pesquisarMinimoIterativo(abb *raiz)
{
  if (raiz != NULL)
    while (raiz->esq != NULL)
      raiz = raiz->esq;
  return raiz;
}

abb *pesquisarMaximo(abb *raiz)
{
  if (raiz == NULL || raiz->dir == NULL)
    return raiz;
  else
    return pesquisarMaximo(raiz->dir);
}

abb *pesquisarMaximoIterativo(abb *raiz)
{
  if (raiz != NULL)
    while (raiz->dir != NULL)
      raiz = raiz->dir;
  return raiz;
}

int inserir(abb **raiz, Tipo valor)
{
  if (*raiz == NULL)
  {
    *raiz = (abb *) malloc(sizeof(abb));
    (*raiz)->info = valor;
    (*raiz)->esq = NULL;
    (*raiz)->dir = NULL;
    return 1;
  }
  else if (valor < (*raiz)->info)
    return inserir(&((*raiz)->esq), valor);
  else if (valor > (*raiz)->info)
    return inserir(&((*raiz)->dir), valor);
  else
    return 0;
}

abb *remover_menor(abb **raiz)
{
     abb *aux = *raiz;
     if (aux->esq == NULL)
     {
        *raiz = (*raiz)->dir;
        return aux;
     }
     else
        return remover_menor(&((*raiz)->esq));
}

void remover_no(abb **raiz)
{
  abb *aux;
  aux = *raiz;
  if ((*raiz)->esq == NULL && (*raiz)->dir == NULL)
    // No tem ambos os filhos
    raiz = NULL;
  else if ((*raiz)->esq == NULL)
    // No tem filho a esquerda
    *raiz = (*raiz)->dir;
  else if ((*raiz)->dir == NULL)
    // No tem filho a direita
    *raiz = (*raiz)->esq;
  else{ // Tem ambos os filhos
    aux = remover_menor(&(*raiz)->dir);
    (*raiz)->info = aux->info;
  }
  free(aux);
}


void remover(abb **raiz, int elem)
{
  if (raiz == NULL)
    printf("Elemento nao encontrado.\n");
  else if (elem == (*raiz)->info)
    remover_no(raiz);
  else if (elem < (*raiz)->info)
    remover(&((*raiz)->esq),elem);
  else 
    remover(&((*raiz)->dir),elem);
}

void emOrdem (abb *raiz)
{
  if (raiz != NULL)
  {
    emOrdem(raiz->esq);
    printf("%i ", raiz->info);
    emOrdem(raiz->dir);
  }
}

void preOrdem (abb *raiz)
{
  if (raiz != NULL)
  {
    printf("%i ", raiz->info);
    preOrdem(raiz->esq);
    preOrdem(raiz->dir);
  }
}

void posOrdem (abb *raiz)
{
  if (raiz != NULL)
  {
    posOrdem(raiz->esq);
    posOrdem(raiz->dir);
    printf("%i ", raiz->info);
  }
}

void porNivel (abb *raiz)
{
    Fila *f;
    abb *aux;
    if (raiz != NULL)
    {
         f = NULL;
         insFila(&f, raiz);
         while (f != NULL)
         {
             aux = remFila(&f);
             if (aux->esq != NULL)
                insFila(&f, aux->esq);
             if (aux->dir != NULL)
                insFila(&f, aux->dir);
             printf("%d ", aux->info);
         }
    }
}

int qtdElementos(abb *raiz)
{
  if (raiz == NULL)
    return 0;
  else
    return 1 + qtdElementos(raiz->esq) + qtdElementos(raiz->dir);
}

int maximo(int a, int b)
{
  return (a > b) ? a : b;
  //if (a > b)
  //  return a;
  //else
  //  return b;
}

int altura(abb *raiz)
{
  if (raiz == NULL || (raiz->esq == NULL && raiz->dir == NULL))
    return 0;
  else
    return 1 + maximo(altura(raiz->esq), altura(raiz->dir));
}

int qtdPares(abb *raiz)
{
  if (raiz == NULL)
    return 0;
  else if (raiz->info % 2 == 0)
    return 1 + qtdPares(raiz->esq) + qtdPares(raiz->dir);
  else
    return 0 + qtdPares(raiz->esq) + qtdPares(raiz->dir);
}

int qtdImpares(abb *raiz)
{
  if (raiz == NULL)
    return 0;
  else if (raiz->info % 2 != 0)
    return 1 + qtdImpares(raiz->esq) + qtdImpares(raiz->dir);
  else
    return 0 + qtdImpares(raiz->esq) + qtdImpares(raiz->dir);
}

int somaNivel(abb *raiz, int nivel)
{
  if (raiz == NULL)
    return 0;
  else if (nivel == 0)
    return raiz->info;
  else
    return somaNivel(raiz->esq, nivel - 1) 
         + somaNivel(raiz->dir, nivel - 1);
}

int mesmaEstruturaAux(abb *a1, abb *a2)
{
  if (a1 == NULL && a2 == NULL)
    return 1;
  else if ((a1 == NULL && a2 != NULL) 
        || (a1 != NULL && a2 == NULL))
    return 0;
  else
    return mesmaEstruturaAux(a1->esq, a2->esq) &&
           mesmaEstruturaAux(a1->dir, a2->dir);
}

int mesmaEstrutura(abb *raiz)
{
  if (raiz == NULL)
    return 1;
  else
    return mesmaEstruturaAux(raiz->esq, raiz->dir);
}